# `String.isBlank()` – 在 Java 中检查空白或空字符串

> 原文： [https://howtodoinjava.com/java11/check-blank-string/](https://howtodoinjava.com/java11/check-blank-string/)

学会使用[`String.isBlank()`](https://docs.oracle.com/en/java/javase/12/docs/api/java.base/java/lang/String.html#isBlank())方法确定给定的字符串是空白还是空，或者仅包含空格。`isBlank()`方法已添加到 [Java 11](https://howtodoinjava.com/java11/features-enhancements/) 中。

要检查给定的字符串是否没有偶数空格，请使用[`String.isEmpty()`](https://howtodoinjava.com/java/string/always-use-length-instead-of-equals-to-check-empty-string-in-java/)方法。

## 1\. `String.isBlank()`方法

如果给定的字符串为空或仅包含空格代码点，则此方法返回`true`，否则返回`false`。

它使用[`Character.isWhitespace(char)`](https://docs.oracle.com/en/java/javase/12/docs/api/java.base/java/lang/Character.html#isWhitespace(int))方法确定空白字符。

```java
/**
* returns 	- true if the string is empty or contains only white space codepoints
*			- otherwise false
*/

public boolean isBlank()

```

## 2\. `String.isBlank()`示例

检查给定字符串是否为空的 Java 程序。

```java
public class Main 
{
	public static void main(String[] args) 
	{
		System.out.println( "ABC".isBlank() );			//false

		System.out.println( " ABC ".isBlank() );		//false

		System.out.println( "  ".isBlank() );			//true

		System.out.println( "".isBlank() );				//true
	}
}

```

程序输出。

```java
false
false
true
true

```

## 3\. `isBlank()`与`isEmpty()`

两种方法都用于检查 Java 中的空白或空字符串。 两种方法之间的区别在于，当且仅当字符串长度为 0 时，`isEmpty()`方法返回`true`。

`isBlank()`方法仅检查非空白字符。 它不检查字符串长度。

```java
public class Main 
{
	public static void main(String[] args) 
	{
		System.out.println( "ABC".isBlank() );		//false
		System.out.println( "  ".isBlank() );		//true

		System.out.println( "ABC".isEmpty() );		//false
		System.out.println( "  ".isEmpty() );		//false
	}
}

```

程序输出：

```java
false
true
false
false

```

学习愉快！