# JUnit5 `@BeforeAll`注解示例

> 原文： [https://howtodoinjava.com/junit5/before-all-annotation-example/](https://howtodoinjava.com/junit5/before-all-annotation-example/)

JUnit5 [`@BeforeAll`](http://junit.org/junit5/docs/current/api/org/junit/jupiter/api/BeforeAll.html) 注解替换了 [JUnit4](//howtodoinjava.com/junit-4/) 中的`@BeforeClass`注解。 它用于表示应在当前测试类中的所有测试之前执行**注解方法**。

## `@BeforeAll`注解用法

使用`@BeforeAll`注解方法，如下所示：

```java
@BeforeAll
public static void init(){
	System.out.println("BeforeAll init() method called");
}

```

`@BeforeAll`注解的方法**必须是[静态](https://howtodoinjava.com/java/basics/java-static-keyword/)方法**，否则它将引发运行时错误。

```java
org.junit.platform.commons.JUnitException: @BeforeAll method 'public void com.howtodoinjava.junit5.examples.JUnit5AnnotationsExample.init()' must be static.
at org.junit.jupiter.engine.descriptor.LifecycleMethodUtils.assertStatic(LifecycleMethodUtils.java:66)
at org.junit.jupiter.engine.descriptor.LifecycleMethodUtils.lambda$findBeforeAllMethods$0(LifecycleMethodUtils.java:42)
at java.util.ArrayList.forEach(ArrayList.java:1249)
at java.util.Collections$UnmodifiableCollection.forEach(Collections.java:1080)
at org.junit.jupiter.engine.descriptor.LifecycleMethodUtils.findBeforeAllMethods(LifecycleMethodUtils.java:42)
```

## `@BeforeAll`注解示例

让我们举个例子。 我使用了一个`Calculator`类并添加了一个`add`方法。 我将使用`@RepeatedTest`注解对其进行 5 次测试。 此注解将导致`add`测试运行 5 次。 但是`@BeforeAll`带注解的方法只能调用一次。

```java
package com.howtodoinjava.junit5.examples;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.RepeatedTest;
import org.junit.jupiter.api.RepetitionInfo;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInfo;
import org.junit.platform.runner.JUnitPlatform;
import org.junit.runner.RunWith;

@RunWith(JUnitPlatform.class)
public class BeforeAllTest {

	@DisplayName("Add operation test")
	@RepeatedTest(5)
	void addNumber(TestInfo testInfo, RepetitionInfo repetitionInfo) {
		System.out.println("Running test -> " + repetitionInfo.getCurrentRepetition());
		Assertions.assertEquals(2, Calculator.add(1, 1), "1 + 1 should equal 2");
	}

	@BeforeAll
	public static void init(){
		System.out.println("Before All init() method called");
	}
}

```

其中`Calculator`类是：

```java
package com.howtodoinjava.junit5.examples;

public class Calculator 
{
	public int add(int a, int b) {
		return a + b;
	}
}

```

现在执行测试，您将看到以下控制台输出：

```java
Before All init() method called
Running test -> 1
Running test -> 2
Running test -> 3
Running test -> 4
Running test -> 5

```

显然，带注解的`@BeforeAll`方法`init()`仅被调用一次。

学习愉快！

[源码下载](https://github.com/lokeshgupta1981/Junit5Examples/tree/master/JUnit5Examples)