# Gradle 教程 – 安装和 HelloWorld 示例

> 原文： [https://howtodoinjava.com/gradle/gradle-tutorial-installation-and-hello-world-example/](https://howtodoinjava.com/gradle/gradle-tutorial-installation-and-hello-world-example/)

[Gradle](https://gradle.org/) 是类似于 Maven 或 Ant 的构建自动化工具，用于编写构建脚本以在任何环境中按需运行它们。 在本教程中，我们将了解 Gradle 与其他构建自动化工具的区别，然后我们将看到一个简单的示例来安装和使用 gradle。

```java
Table of Contents

Gradle vs. other build tools
Installing Gradle
Writing simple Gradle script
Executing Gradle script
```

![Gradle](img/5718f5e4d6df73ba162961e21a815228.png)

## Gradle 与其他构建工具

如果您使用过 maven，则知道您不能从一个构建文件（`pom.xml`）生成多个工件（jar 或 war 文件等）。 如果要从单个`src`文件夹生成两个 jar 文件，则可能必须在 maven 中创建两个项目。 这是现有构建工具的困难之一。

多年来，其他构建工具已用于编译和打包软件。 但是随着不断变化的行业需求，项目涉及大量多样的软件栈，合并多种编程语言并应用广泛的测试策略。 随着敏捷实践的兴起，构建必须支持代码的早期集成以及频繁且轻松地交付给测试和生产环境。

其他工具不足以实现这些目标，而 Gradle 正是其中之一。 它借鉴了从既有工具中汲取的经验教训，并将其最佳思想带入了一个新的水平。

Gradle 管理依赖项的能力不仅限于外部库。 随着项目规模和复杂性的增长，您将需要将代码组织到职责明确的模块中。 Gradle 为定义和组织多项目构建以及对项目之间的依赖项建模提供了强大的支持。

Gradle 构建脚本具有声明性，可读性，并且清楚地表达了其意图。 用 Groovy 而不是 XML 编写代码，再加上 Gradle 的按惯例构建理念，大大减少了构建脚本的大小，并且可读性更高。

## 安装 Gradle

1.  作为前提条件，请确保您已经安装了 JDK 6 或更高版本。
2.  Gradle 入门很容易。 您可以直接从 Gradle 主页上的 [https://gradle.org/gradle-download/](https://gradle.org/gradle-download/) 下载该发行版。 发行版附带源代码，二进制文件，文档和示例。
3.  在某个位置将下载的 zip 文件解压缩到开发计算机中。 现在，将环境变量`GRADLE_HOME`设置为 gradle 目录。
4.  将`GRADLE_HOME/bin` 文件夹添加到`PATH`环境变量。

您应该能够从命令行运行`gradle -v`命令。 如果发生这种情况，并且您获得了输出，就可以在项目中使用 gradle 了。 上面命令的输出将如下所示：

```java
------------------------------------------------------------
Gradle 2.14.1
------------------------------------------------------------

Build time:   2016-07-18 06:38:37 UTC
Revision:     d9e2113d9fb05a5caabba61798bdb8dfdca83719

Groovy:       2.4.4
Ant:          Apache Ant(TM) version 1.9.6 compiled on June 29 2015
JVM:          1.6.0_33 (Sun Microsystems Inc. 20.8-b03)
OS:           Windows 7 6.1 x86
```

## 编写简单的 Gradle 脚本

现在，让我们编写一个简单的 gradle 脚本并尝试运行它。 我已经在桌面上创建了新文件夹，并以`Playground`命名。 然后，我创建了`build.gradle`并写下了 HelloWorld 脚本。

```java
task helloWorld {
   doLast {
      println 'Hello world!'
   }
}

```

上面的脚本是用 Gradle 的 DSL（自我描述性语言）编​​写的，例如，名为`doLast`的动作几乎可以自我表达。 这是为任务执行的最后一个动作。

## 执行 Gradle 脚本

要运行脚本，请运行`gradle`命令并传递任务名称。 输出将如下所示：

```java
> gradle –q helloWorld
Hello world!

```

在`–q`中使用安静选项时，您告诉 Gradle 仅输出任务的输出。

上面的用例本质上是非常基本的，并且其编写目的是使事情尽可能简单 — 因此您首先要熟悉最基本的东西。 在接下来的教程中，我们将学习 Gradle 的更高级功能。

学习愉快！