# Log4j2 `ThreadContext` – 相同事务的鱼标日志

> 原文： [https://howtodoinjava.com/log4j2/threadcontext-fish-tagging/](https://howtodoinjava.com/log4j2/threadcontext-fish-tagging/)

Log4j2 [`ThreadContext`](https://logging.apache.org/log4j/2.0/log4j-api/apidocs/org/apache/logging/log4j/ThreadContext.html)允许您使用多个唯一的标签标记日志语句，以分析日志，同时在运行时诊断问题 - 主要是在多线程应用中，其中应用会在短时间内生成大量日志。 例如。 您可能需要扫描所有日志以查找特定的用户事务或完成会话。 此过程也称为鱼标（即，在每个日志语句中添加一些额外的上下文信息）。 鱼标可以帮助使用自动化工具记录日志，例如 [Splunk](http://www.splunk.com/) 。

让我们看看如何在 log4j2 中将`ThreadContext`类用于[鱼标](https://logging.apache.org/log4j/2.x/manual/thread-context.html)。

## 在`ThreadContext`中添加上下文信息

为了唯一标记每个请求，`ThreadContext`提供了`put(String key, String value)`方法，该方法接受键及其值。 您可以根据需要添加任意数量的标签以捕获整个上下文信息。 请注意，`ThreadContext`类的所有方法都是静态的。

```java
package com.howtodoinjava.log4j2.examples;

import java.util.UUID;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.ThreadContext;

public class Log4j2HelloWorldExample 
{
	private static final Logger LOGGER = LogManager.getLogger(Log4j2HelloWorldExample.class.getName());

	public static void main(String[] args) 
	{
		//Add context information
		ThreadContext.put("id", UUID.randomUUID().toString());
		ThreadContext.put("ipAddress", "192.168.21.9");

		LOGGER.debug("Debug Message Logged !!");
		LOGGER.info("Info Message Logged !!");
		LOGGER.debug("Another Debug Message !!");

		//Clear the map
		ThreadContext.clearMap();

		LOGGER.debug("Thread Context Cleaned up !!");
		LOGGER.debug("Log message with no context information !!");
	}
}

```

或者，您可以将`ThreadContext`的`Stack`实现与`ThreadContext.push(String value)`协作，如下所示：

```java
//Add context information
ThreadContext.push(UUID.randomUUID().toString());
ThreadContext.push("192.168.21.9");

LOGGER.debug("Debug Message Logged !!");
LOGGER.info("Info Message Logged !!");
LOGGER.debug("Another Debug Message !!");

//Clear the map
ThreadContext.clearStack();

LOGGER.debug("Thread Context Cleaned up !!");
LOGGER.debug("Log message with no context information !!");

```

事务处理结束或不再需要上下文信息后，可以使用`ThreadContext.clearMap()`方法清空信息。

默认情况下，`ThreadContext`的栈和映射是按线程管理的，并且基于[`ThreadLocal`](//howtodoinjava.com/java/multi-threading/when-and-how-to-to-to-use-thread-local-variables/)。 通过将系统属性`isThreadContextMapInheritable`设置为`true`，上下文映射的内容将传递给子线程。

## 修改`log4j2.xml`中的转换模式

现在要在日志语句中打印以上标记，您需要在 log4j2 配置文件中修改转换模式。

```java
<?xml version="1.0" encoding="UTF-8"?>
<Configuration status="INFO">
	<Appenders>
		<Console name="console" target="SYSTEM_OUT">
			<PatternLayout
				pattern="[%-5level] %d{yyyy-MM-dd HH:mm:ss.SSS} [%t] [%X{id}] [%X{ipAddress}] %c{1} - %msg%n" />
		</Console>
	</Appenders>
	<Loggers>
		<Root level="debug" additivity="false">
			<AppenderRef ref="console" />
		</Root>
	</Loggers>
</Configuration>

```

*   单独使用`%X`可以包含映射的全部内容。
*   使用`%X{key}`包含指定的键。
*   使用`%x`包含栈的全部内容。

现在，当您运行上面的代码时 – 您将获得以下输出：

```java

[DEBUG] 2016-06-21 13:09:56.485 [main] [7cdd4cf0-2c26-4b81-b374-1adce3781499] [192.168.21.9] Log4j2HelloWorldExample - Debug Message Logged !!
[INFO ] 2016-06-21 13:09:56.487 [main] [7cdd4cf0-2c26-4b81-b374-1adce3781499] [192.168.21.9] Log4j2HelloWorldExample - Info Message Logged !!
[DEBUG] 2016-06-21 13:09:56.487 [main] [7cdd4cf0-2c26-4b81-b374-1adce3781499] [192.168.21.9] Log4j2HelloWorldExample - Another Debug Message !!
[DEBUG] 2016-06-21 13:09:56.487 [main] [] [] Log4j2HelloWorldExample - Thread Context Cleaned up !!
[DEBUG] 2016-06-21 13:09:56.487 [main] [] [] Log4j2HelloWorldExample - Log message with no context information !!

```

如您所见，前三个日志语句中添加了上下文信息 - 其他两个语句没有此类信息。

学习愉快！