# `log4j.properties`示例 – Log4j 属性文件示例

> 原文： [https://howtodoinjava.com/log4j/how-to-configure-log4j-using-properties-file/](https://howtodoinjava.com/log4j/how-to-configure-log4j-using-properties-file/)

[**Log4j**](https://howtodoinjava.com/log4j/ "log4j") 是一个简单而灵活的日志记录框架。 日志记录为开发人员提供了有关应用故障的详细上下文。 使用 log4j 可以在运行时启用日志记录，而无需修改应用二进制文件。 log4j 包的设计目的是使这些语句可以保留在出厂代码中，而不会造成高昂的性能成本。

这篇 **log4j 属性文件教程**，我展示了使用`log4j.properties`文件配置 log4j 的示例代码。

## 1\. Log4j Maven 依赖项

[创建一个 maven java 项目](https://howtodoinjava.com/maven/create-java-project-maven/)并更新 log4j maven 依赖项。

```java
<dependency>
    <groupId>log4j</groupId>
    <artifactId>log4j</artifactId>
    <version>1.2.17</version>
</dependency>

```

## 2\. `log4j.properties`文件

这是主要属性文件，具有 log4j 使用的所有运行时配置。 该文件将具有 log4j 附加器信息，日志级别信息和文件附加器的输出文件名。

```java
log4j.rootLogger=DEBUG, consoleAppender, fileAppender

log4j.appender.consoleAppender=org.apache.log4j.ConsoleAppender
log4j.appender.consoleAppender.layout=org.apache.log4j.PatternLayout
log4j.appender.consoleAppender.layout.ConversionPattern=[%t] %-5p %c %x - %m%n

log4j.appender.fileAppender=org.apache.log4j.RollingFileAppender
log4j.appender.fileAppender.layout=org.apache.log4j.PatternLayout
log4j.appender.fileAppender.layout.ConversionPattern=[%t] %-5p %c %x - %m%n
log4j.appender.fileAppender.File=demoApplication.log
```

## 3\. `log4j.properties`示例

```java
package com.howtodoinjava;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

public class Log4jPropertiesConfigurationExample
{
	static Logger logger = Logger.getLogger(Log4jPropertiesConfigurationExample.class);
	public static void main(String[] args)
	{
		//PropertiesConfigurator is used to configure logger from properties file
		PropertyConfigurator.configure("log4j.properties");

		//Log in console in and log file
		logger.debug("Log4j appender configuration is successful !!");
	}
}

```

在控制台中输出，在项目根文件夹中输出`demoApplication.log`：

```java
[main] DEBUG com.howtodoinjava.Log4jPropertiesConfigurationExample&nbsp; - Log4j appender configuration is successful !!

```

现在，我们来看一些`log4j.properties`示例，用于将日志消息输出到特定位置。

## 4\. Log4j `ConsoleAppender` – 记录到控制台

将日志输出到控制台的 Java 程序。

```java
# Root logger
log4j.rootLogger=INFO, console

log4j.appender.console=org.apache.log4j.ConsoleAppender
log4j.appender.console.Target=System.out
log4j.appender.console.layout=org.apache.log4j.PatternLayout
log4j.appender.consoleAppender.layout.ConversionPattern=[%t] %-5p %c %x - %m%n

```

## Log4j `RollingFileAppender` – 记录到文件

将日志输出到文件的 Java 程序。

```java
# Root logger
log4j.rootLogger=INFO, file

# Direct log messages to a log file
log4j.appender.file=org.apache.log4j.RollingFileAppender

log4j.appender.file.File=C:\\temp\info.log
log4j.appender.file.MaxFileSize=10MB
log4j.appender.file.MaxBackupIndex=10
log4j.appender.file.layout=org.apache.log4j.PatternLayout
log4j.appender.file.layout.ConversionPattern=[%t] %-5p %c %x - %m%n

```

让我知道是否有任何问题。

学习愉快！