# Python 教程

> 原文： [https://howtodoinjava.com/python-tutorial/](https://howtodoinjava.com/python-tutorial/)

Python 是一种流行的编程语言，由 Guido van Rossum 创建，并于 1991 年发布。Python 被认为是最流行的编程语言中最热门的技能之一。

它是开源的，即我们甚至可以出于商业目的自由安装，使用和分发。

在本教程中，我们将学习 python 基础知识和一些高级概念。

## 1. Python 是解释语言

编程语言通常分为两类 - **解释语言**和**编译语言**。

*编译语言*是指使用编译器事先将源代码编译为可执行指令的语言（例如 Java）。 以后，这些符合条件的指令可以由运行时环境执行。

*解释语言*是不应用中间编译步骤并且可以将源代码直接提供给运行时环境的语言。 在此，*源代码到机器代码的转换*在执行器的同时发生。

意味着，任何用 python 编写的源代码都可以直接执行而无需编译。

## 2\. Python 很简单

Python 主要是为了强调代码的可读性而开发的，它的语法允许程序员用更少的代码行来表达概念。

根据语言中可用关键字的简单性粗略衡量，Python3 有 33 个关键字，Python 2 有 31 个关键字。相比之下，C++ 有 62 个关键字，Java 有 53 个关键字。

Python 语法提供了一种易于学习和易于阅读的简洁结构。

## 3.与其他语言比较

*   Python 使用*新行来完成语句*。 在其他编程语言中，我们经常使用分号或括号。
*   Python 依靠缩进（使用空格）来定义范围，例如，循环，函数和类。 为此，其他编程语言通常使用花括号。

## 4.用途和好处

Python 可用于快速原型制作或可用于生产环境的软件开发。 以下列表列出了 python 的一些流行用法。

*   Python 具有强大的标准库和许多有用的模块，可用于开发应用。 这些模块可以帮助我们添加所需的功能，而无需编写更多代码。
*   由于 python 是一种解释型高级编程语言，它使我们无需修改即可在多个平台上运行相同的代码。
*   Python 可用于以程序样式，面向对象样式或功能样式编写应用。
*   Python 具有分析数据和可视化等功能，有助于为*大数据分析*，*机器学习*和*人工智能*创建自定义解决方案。
*   Python 还用于机器人技术，网页抓取，脚本编写，人脸检测，颜色检测和 3D 应用中。 我们可以使用 python 构建基于控制台的应用，基于音频的应用，基于视频的应用，企业应用等。

## 5.安装 Python

如今，大多数计算机和操作系统均已安装了 python。 要检查机器中是否已经存在 python，请执行以下命令。

```py
$ python --version

#prints

Python3.8.0

```

如果机器没有安装 python，那么我们可以从以下网站免费下载它：<https://www.python.org/>。

## 6.编写并执行 python 代码

#### 6.1 Python 文件

如前所述，python 是一种解释型语言，因此我们可以在扩展名为（`.py`）的文件中编写源代码，并使用`'python'`命令执行该文件。

让我们在任何文本编辑器中编写第一个 Python 文件`helloworld.py`。

```py
print("Hello, World!")

```

保存文件并在命令提示符或控制台中执行它。

```py
$ python helloworld.py

#prints

Hello, World!

```

#### 6.2 内联代码

Python 代码可以直接在命令行中运行，通常对于测试少量代码很有用。

要获取 python 控制台，请在 OS 控制台中键入命令`'python'`。

```py
$ python

Python3.8.0 (tags/v3.8.0:fa919fd, Oct 14 2019, 19:21:23) [MSC v.1916 32 bit (Intel)] on win32
Type "help", "copyright", "credits" or "license" for more information.

>>> print("Hello, World!")

Hello, World!

```

## 7.学习 Python

#### 7.1 语言基础

[Python – 在 Sublime 编辑器中安装](https://howtodoinjava.com/python/install-python-sublime-editor/)

[Python – 注释](https://howtodoinjava.com/python/python-comments/)

[Python – 变量](https://howtodoinjava.com/python/python-variables/)

[Python – 数据类型](https://howtodoinjava.com/python/python-data-types/)

[Python – 关键字](https://howtodoinjava.com/python/python-keywords/)

[Python – 关键字](https://howtodoinjava.com/python/python-keywords/)

[Python – 整数](https://howtodoinjava.com/python/python-integer-ints/)

[Python – 字符串](https://howtodoinjava.com/python/python-strings/)

[Python – 列表](https://howtodoinjava.com/python/python-lists/)

[Python – 元组](https://howtodoinjava.com/python/python-tuples/)

#### 7.2 字符串

[Python – 字符串`split()`示例](https://howtodoinjava.com/python/split-string/)

[Python – 字符串`startswith()`示例](https://howtodoinjava.com/python/string-startswith/)

[Python – 字符串`endswith()`示例](https://howtodoinjava.com/python/string-endswith-method/)

#### 7.3 集合

[Python – 查找列表或数组中的最大值和最小值](https://howtodoinjava.com/python/max-min/)

[Python – 查找最大 N 个（前 N 个）或最小 N 个项](https://howtodoinjava.com/python/find-largest-smallest-items/)

[Python- 将元组解包为变量](https://howtodoinjava.com/python/unpack-tuple-sequence/)

[Python – 元组比较](https://howtodoinjava.com/python/compare-tuples/)

[Python – 列表与元组](https://howtodoinjava.com/python/lists-vs-tuples/)

[Python – 解包元组 – 太多值无法解包](https://howtodoinjava.com/python/unpack-variable-length-tuple/)

[Python – `Multidict`示例](https://howtodoinjava.com/python/multidict-key-to-multiple-values/)

[Python – `OrderedDict`示例](https://howtodoinjava.com/python/ordereddict-ordered-dictionary/)

[Python – 字典交集](https://howtodoinjava.com/python/dictionary-intersection/)

[Python – 优先级队列示例](https://howtodoinjava.com/python/priority-queue/)

#### 7.4 杂项

[Python – 读写 CSV 文件](https://howtodoinjava.com/python/python-read-write-csv-files/)

[Python – `httplib2`库](https://howtodoinjava.com/python/httplib2-http-get-post-requests/)

在评论中向我发送有关 python 的问题。

学习愉快！