# Jackson – 将 JSON 转换为`Map`并将`Map`转换为 JSON

> 原文： [https://howtodoinjava.com/jackson/jackson-json-to-from-hashmap/](https://howtodoinjava.com/jackson/jackson-json-to-from-hashmap/)

在 **[Jackson](https://howtodoinjava.com/jackson/jackson-examples-convert-java-object-to-from-json/) json 映射示例**示例中，我们将学习**将 json 转换为映射对象**，然后我们将学习**将 java 映射转换为 json**。

## 1\. Jackson 依赖项

在您的应用项目中包括 **Jackson2** 依赖项。 不要忘记在 maven 站点检查[最新依赖项](https://mvnrepository.com/artifact/com.fasterxml.jackson.core/jackson-databind/2.9.6)。

```java
<dependency>
    <groupId>com.fasterxml.jackson.core</groupId>
    <artifactId>jackson-databind</artifactId>
    <version>2.9.6</version>
</dependency>

```

## 2\. Jackson 将`Map`转换为 JSON

将`Map`转换为 JSON 的 Java 程序如下。 我在这里使用[**`HashMap`**](https://howtodoinjava.com/java/collections/how-hashmap-works-in-java/)。

```java
package com.howtodoinjava.jackson2.example;

import java.io.IOException;
import java.util.HashMap;

import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

public class MapToJSON 
{
	public static void main(String[] args) 
	{
		HashMap<String, Object> hashmap = new HashMap<String, Object>();

		hashmap.put("id", 11);
		hashmap.put("firstName", "Lokesh");
		hashmap.put("lastName", "Gupta");

		ObjectMapper mapper = new ObjectMapper();
		try 
		{
			//Convert Map to JSON
			String json = mapper.writeValueAsString(hashmap);

			//Print JSON output
			System.out.println(json);
		} 
		catch (JsonGenerationException e) {
			e.printStackTrace();
		} catch (JsonMappingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}

```

程序输出。

```java
{"id":11,"lastName":"Gupta","firstName":"Lokesh"}

```

## 3\. Jackson 将 JSON 转换为`Map`

将 JSON 转换为`Map`的 Java 程序如下。 我在这里使用`HashMap`。

```java
package com.howtodoinjava.jackson2.example;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

public class JSONToMap
{
	public static void main(String[] args) 
	{
		String json = "{\"id\":1,\"name\":\"Lokesh Gupta\",\"age\":34,\"location\":\"India\"}";

		HashMap<String, Object> map = new HashMap<String, Object>();

		ObjectMapper mapper = new ObjectMapper();
		try 
		{
			//Convert Map to JSON
			map = mapper.readValue(json, new TypeReference<Map<String, Object>>(){});

			//Print JSON output
			System.out.println(map);
		} 
		catch (JsonGenerationException e) {
			e.printStackTrace();
		} catch (JsonMappingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}

```

程序输出：

```java
{id=1, name=Lokesh Gupta, age=34, location=India}

```

学习愉快！